@@ -13,6 +13,12 @@ miquire :lib, 'typed-array', 'timelimitedqueue'
 投稿１つを表すクラス。
 =end
 class Message < Retriever::Model
+  # screen nameにマッチする正規表現
+  MentionMatcher      = /(?:@|＠|〄|☯|⑨|♨|(?:\W|^)D )([a-zA-Z0-9_]+)/.freeze
+
+  # screen nameのみから構成される文字列から、@などを切り取るための正規表現
+  MentionExactMatcher = /\A(?:@|＠|〄|☯|⑨|♨|D )?([a-zA-Z0-9_]+)\Z/.freeze
+
   @@system_id = 0
   @@appear_queue = TimeLimitedQueue.new(65536, 0.1, Set){ |messages|
     Plugin.call(:appear, messages) }
@@ -102,8 +108,9 @@ class Message < Retriever::Model
 
   # この投稿のお気に入り状態を返す。お気に入り状態だった場合にtrueを返す
   def favorite?
-    favorited_by.include?(Service.primary.user_obj)
-  end
+    favorited_by.include?(Service.primary!.user_obj)
+  rescue Service::NotExistError
+    false end
 
   # 投稿がシステムメッセージだった場合にtrueを返す
   def system?
@@ -136,13 +143,7 @@ class Message < Retriever::Model
 
   # この投稿が自分宛ならばtrueを返す
   def to_me?
-    return true if self.system?
-    if self.service
-      return true if self.receive_to?(self.service.user_obj)
-      return true if self[:message].to_s.include?(self.service.user.to_s)
-    end
-    false
-  end
+    system? or Service.map(&:user_obj).find(&method(:receive_to?)) end
 
   # この投稿の投稿主を返す
   def user
@@ -167,31 +168,23 @@ class Message < Retriever::Model
       self[:receiver] = parallel{
         self[:receiver] = User.findbyid(receiver_id) }
     else
-      match = (/@([a-zA-Z0-9_]+)/).match(self[:message].to_s)
+      match = MentionMatcher.match(self[:message].to_s)
       if match
         result = User.findbyidname(match[1])
         self[:receiver] = result if result end end end
-  memoize :receiver
 
   # ユーザ _other_ に宛てられたメッセージならtrueを返す。
   # _other_ は、 User か_other_[:id]と_other_[:idname]が呼び出し可能なもの。
   def receive_to?(other)
     type_strict other => :[]
-    if self[:receiver].is_a? User
-      other[:id] == self[:receiver][:id]
-    elsif self[:receiver]
-      other[:id] == self[:receiver]
-    else
-      match = (/@([a-zA-Z0-9_]+)/).match(self[:message].to_s)
-      if match
-        match[1] == other[:idname] end end end
+    (self[:receiver].is_a?(User) and other[:id] == self[:receiver][:id]) or receive_user_screen_names.include? other[:idname] end
 
   # このツイートが宛てられたユーザを可能な限り推測して、その idname(screen_name) を配列で返す。
   # 例えばツイート本文内に「@a @b @c」などと書かれていたら、["a", "b", "c"]を返す。
   # ==== Return
   # 宛てられたユーザの idname(screen_name) の配列
   def receive_user_screen_names
-    self[:message].to_s.to_enum(:each_matches, /@([a-zA-Z0-9_]+)/).map{ |m| m[1] } end
+    self[:message].to_s.scan(MentionMatcher).map(&:first) end
 
   # 自分がこのMessageにリプライを返していればtrue
   def mentioned_by_me?
@@ -232,7 +225,7 @@ class Message < Retriever::Model
     result.add_child(this) unless result.children.include?(this) }
 
   # Message#receive_message と同じ。ただし、ReTweetedのみをさがす。
-  define_source_getter(:retweet, /^RT/){ |this, result|
+  define_source_getter(:retweet, -> t { t.start_with? 'RT'.freeze }){ |this, result|
     result.add_child(this) unless result.retweeted_statuses.include?(this) }
 
   # 投稿の宛先になっている投稿を再帰的にさかのぼり、それぞれを引数に取って
@@ -282,8 +275,13 @@ class Message < Retriever::Model
 
   # この投稿を「自分」がふぁぼっていれば真
   def favorited_by_me?(me = Service.services)
-    not (Set.new(favorited_by.map(&:idname)) & Set.new(me.map(&:idname))).empty?
-  end
+    case me
+    when Service
+      favorited_by.include? me.user_obj
+    when Enumerable
+      not (Set.new(favorited_by.map(&:idname)) & Set.new(me.map(&:idname))).empty?
+    else
+      raise ArgumentError, "first argument should be `Service' or `Enumerable'. but given `#{me.class}'" end end
 
   # この投稿をリツイートしたユーザを返す
   def retweeted_by
@@ -296,12 +294,19 @@ class Message < Retriever::Model
 
   # 選択されているユーザがこのツイートをリツイートしているなら真
   def retweeted?
-    retweeted_by.include?(Service.primary.user_obj) end
+    retweeted_by.include?(Service.primary!.user_obj)
+  rescue Service::NotExistError
+    false end
 
   # この投稿を「自分」がリツイートしていれば真
   def retweeted_by_me?(me = Service.services)
-    not (Set.new(retweeted_by.map(&:idname)) & Set.new(me.map(&:idname))).empty?
-  end
+    case me
+    when Service
+      retweeted_by.include? me.user_obj
+    when Enumerable
+      not (Set.new(retweeted_by.map(&:idname)) & Set.new(me.map(&:idname))).empty?
+    else
+      raise ArgumentError, "first argument should be `Service' or `Enumerable'. but given `#{me.class}'" end end
 
   # 非公式リツイートやハッシュタグを適切に組み合わせて投稿する
   def body
@@ -431,7 +436,7 @@ class Message < Retriever::Model
     attr_accessor :url
     attr_reader :resource
 
-    IS_URL = /^https?:\/\//
+    IS_URL = /\Ahttps?:\/\//
 
     def initialize(resource)
       if(not resource.is_a?(IO)) and (FileTest.exist?(resource.to_s)) then
