@@ -11,7 +11,6 @@ miquire :mui, 'intelligent_textview'
 
 module Gtk
   class PostBox < Gtk::EventBox
-
     attr_accessor :return_to_top
 
     @@ringlock = Mutex.new
@@ -22,12 +21,13 @@ module Gtk
       return @@postboxes
     end
 
-    def initialize(watch, options = {})
+    def initialize(postable = PostToPrimaryService.new, options = {})
+      type_strict postable => :post, options => Hash
       mainthread_only
       @posting = nil
       @return_to_top = nil
       @options = options
-      @watch = watch
+      @postable = postable
       super()
       signal_connect('parent-set'){
         if parent
@@ -66,12 +66,9 @@ module Gtk
       post_set_default_text(@post)
       @post.wrap_mode = Gtk::TextTag::WRAP_CHAR
       @post.border_width = 2
-      @post.ssc('key_release_event'){ |textview, event|
+      @post.buffer.ssc('changed') { |textview|
         refresh_buttons(false)
         false }
-      @post.ssc('paste-clipboard'){ |this|
-        Delayer.new{ refresh_buttons(false) }
-        false }
       @post.signal_connect_after('focus_out_event', &method(:focus_out_event))
       @post end
     alias post widget_post
@@ -82,9 +79,7 @@ module Gtk
       Delayer.new{
         if not @remain.destroyed?
           @remain.set_text(remain_charcount.to_s) end }
-      widget_post.ssc('key_release_event'){ |textview, event|
-        @remain.set_text(remain_charcount.to_s) }
-      widget_post.ssc('paste-clipboard'){ |this|
+      widget_post.buffer.ssc(:changed){ |textview, event|
         @remain.set_text(remain_charcount.to_s) }
       @remain end
 
@@ -138,7 +133,6 @@ module Gtk
         text = widget_post.buffer.text
         text += UserConfig[:footer] if add_footer?
         @posting = service.post(:message => text){ |event, msg|
-          notice [event, msg].inspect
           case event
           when :start
             Delayer.new{ start_post }
@@ -169,7 +163,7 @@ module Gtk
       if(@options[:postboxstorage])
         return false if delegate
         if not @options[:delegated_by]
-          postbox = Gtk::PostBox.new(@watch, @options)
+          postbox = Gtk::PostBox.new(@postable, @options)
           @options[:postboxstorage].
             pack_start(postbox).
             show_all.
@@ -208,18 +202,18 @@ module Gtk
         options = @options.clone
         options[:delegate_other] = false
         options[:delegated_by] = self
-        @options[:postboxstorage].pack_start(Gtk::PostBox.new(@watch, options)).show_all
+        @options[:postboxstorage].pack_start(Gtk::PostBox.new(@postable, options)).show_all
         true end end
 
     def service
       if UserConfig[:legacy_retweet_act_as_reply]
-        @watch
+        @postable
       else
-        (retweet? ? @watch.service : @watch) end end
+        (retweet? ? @postable.service : @postable) end end
 
     def post_is_empty?
       widget_post.buffer.text.empty? or
-        (defined?(@watch[:user]) ? widget_post.buffer.text == "@#{@watch[:user][:idname]} " : false) end
+        (defined?(@postable[:user]) ? widget_post.buffer.text == "@#{@postable[:user][:idname]} " : false) end
 
     def brothers
       if(@options[:postboxstorage])
@@ -251,7 +245,7 @@ module Gtk
         @on_delete.call end end
 
     def reply?
-      @watch.is_a?(Retriever::Model) end
+      @postable.is_a?(Retriever::Model) end
 
     def retweet?
       @options[:retweet] end
@@ -274,7 +268,8 @@ module Gtk
         140 - widget_post.buffer.text.size - footer end end
 
     def focus_out_event(widget, event=nil)
-      Delayer.new(Delayer::NORMAL, @options){ |options|
+      options = @options
+      Delayer.new{
         if(not(frozen?) and not(options.has_key?(:postboxstorage)) and post_is_empty?)
           destroy_if_necessary(widget_send, widget_tool, *@replies) end }
       false end
@@ -301,18 +296,18 @@ module Gtk
         post.buffer.text = @options[:delegated_by].post.buffer.text
         @options[:delegated_by].post.buffer.text = ''
       elsif retweet?
-        post.buffer.text = " RT @" + @watch.idname + ": " + @watch.to_show
+        post.buffer.text = " RT @" + @postable.idname + ": " + @postable.to_show
         post.buffer.place_cursor(post.buffer.start_iter)
       elsif reply?
         post.buffer.text = reply_users + ' ' + post.buffer.text end
       post.accepts_tab = false end
 
     def reply_users
-      replies = [@watch.idname]
+      replies = [@postable.idname]
       if(@options[:subreplies].is_a? Enumerable)
         replies += @options[:subreplies].map{ |m| m.to_message.idname } end
       if @options[:exclude_myself]
-        replies = replies.select{|x| x != @watch.service.idname }
+        replies = replies.select{|x| x != @postable.service.idname }
       end
       replies.uniq.map{ |x| "@#{x}" }.join(' ')
     end
@@ -321,10 +316,20 @@ module Gtk
     def in_reply_to_all
       result = Set.new
       if reply?
-        result << @watch
+        result << @postable
         if @options[:subreplies].is_a? Enumerable
           result += @options[:subreplies] end end
       result end
 
+    class PostToPrimaryService
+      def post(*args, &proc)
+        Service.primary.post(*args, &proc)
+      end
+
+      def service
+        self
+      end
+    end
+
   end
 end
